/* list.h by Jan Bobrowski */

typedef struct list {
	struct list *next, *prev;
} list_t;

static inline void list_link(struct list *a, struct list *b)
{
	a->next = b;
	b->prev = a;
}

static inline void list_add(struct list *head, struct list *item)
{
	struct list *first = head->next;
	list_link(head, item);
	list_link(item, first);
}

static inline void list_add_end(struct list *head, struct list *item)
{
	struct list *last = head->prev;
	list_link(item, head);
	list_link(last, item);
}

static inline void list_del(struct list *item)
{
	list_link(item->prev, item->next);
}

static inline void list_init(struct list *head)
{
	list_link(head, head);
}

static inline void list_del_init(struct list *item)
{
	struct list *prev = item->prev, *next = item->next;
	list_link(item, item);
	list_link(prev, next);
}

static inline int list_empty(struct list *head)
{
	return head->next == head;
}

#define LIST(L) struct list L = {&L, &L}

#define list_entry(ptr, type, member) \
	((type*)((char*)(ptr) - (long)(&((type*)0)->member)))
